@@ -103,6 +103,7 @@ CONFIG cf_options[] = {
   { cft_strg, "bmp-table",	NULL,		NULL,NULL },
   { cft_strg, "bmp-colors",	NULL,		NULL,NULL },
   { cft_strg, "bmp-timer",	NULL,		NULL,NULL },
+  { cft_flag, "large-memory",	NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
 CONFIG cf_all[] = {
@@ -116,6 +117,7 @@ CONFIG cf_all[] = {
   { cft_flag, "lock",		NULL,		NULL,NULL },
   { cft_strg, "fallback",	NULL,		NULL,NULL },
   { cft_flag, "single-key",	NULL,		NULL,NULL },
+  { cft_strg, "literal",	NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
 CONFIG cf_kernel[] = {
@@ -125,7 +127,6 @@ CONFIG cf_kernel[] = {
   { cft_flag, "read-only",	NULL,		NULL,NULL },
   { cft_flag, "read-write",	NULL,		NULL,NULL },
   { cft_strg, "append",		NULL,		NULL,NULL },
-  { cft_strg, "literal",	NULL,		NULL,NULL },
   { cft_strg, "initrd",		NULL,		NULL,NULL },
   { cft_link, NULL,		&cf_all,	NULL,NULL }};
 
@@ -182,6 +183,13 @@ CONFIG cf_change_dsc[] = {
   { cft_flag, "deactivate",	NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
+CONFIG cf_bitmap[] = {
+  { cft_strg, "bitmap",		NULL,		NULL,NULL },
+  { cft_strg, "bmp-table",	NULL,		NULL,NULL },
+  { cft_strg, "bmp-colors",	NULL,		NULL,NULL },
+  { cft_strg, "bmp-timer",	NULL,		NULL,NULL },
+  { cft_end,  NULL,		NULL,		NULL,NULL }};
+
 
 #if NEW_PARSE
 
@@ -218,7 +226,7 @@ void cfg_error(char *msg,...)
     vfprintf(errstd,msg,ap);
     va_end(ap);
     if (!file_name) fputc('\n',errstd);
-    else fprintf(errstd," at or above line %d in file %s\n",line_num,file_name);
+    else fprintf(errstd," at or above line %d in file '%s'\n",line_num,file_name);
     exit(1);
 }
 
@@ -602,3 +610,13 @@ FILE *cfg_pw_open(void)
 #endif    
     return pw_file;
 }
+
+
+/* allow only the "bitmap" keywords */
+
+void cfg_bitmap_only(void)
+{
+    keywords[0] = cf_bitmap;
+    keywords[1] = NULL;
+}
+
