@@ -1,7 +1,7 @@
 /* lilo.c  -  LILO command-line parameter processing */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -35,6 +35,8 @@ source directory.
 #include "partition.h"
 #include "probe.h"
 #include "md-int.h"
+#include "edit.h"
+#include "flags.i"
 
 static int lowest;
 static md_array_info_t md_array_info;
@@ -45,7 +47,6 @@ static char *raid_mbr[MAX_RAID];
 static int raid_device[MAX_RAID+1];
 static int raid_bios[MAX_RAID+1];
 static int device, md_bios;
-static enum {X_NULL=0, X_NONE, X_AUTO, X_MBR_ONLY, X_SPEC} extra;
 enum {MD_NULL=0, MD_PARALLEL, MD_MIXED, MD_SKEWED};
 int do_md_install;
 char *config_file;		/* actual name of the config file */
@@ -543,7 +544,8 @@ static void show_other(int fd)
 	  drvmap[1]);
 	drvmap += 2;
     }
-    while (prtmap[0] && prtmap[1]) {
+    /* fix VERY old bug -- offset of 0 in PT is okay */
+    while (prtmap[0] /*** && prtmap[1] ***/ ) {
 	printf("    BIOS drive 0x%02x, offset 0x%x: 0x%02x -> 0x%02x\n",
 	  prtmap[0],prtmap[1]+PART_TABLE_OFFSET,prtmap[2],prtmap[3]);
 	prtmap += 4;
@@ -605,6 +607,8 @@ static void show_images(char *map_file)
 	else printf("  Enter boot prompt only on demand\n");
 	printf("  Boot-time BIOS data%s saved\n",
 		boot.par_1.prompt & FLAG_NOBD ? " NOT" : "");
+	printf("  Large memory (>15M) is%s used to load initial ramdisk\n", 
+		boot.par_1.prompt & FLAG_LARGEMEM ? "" : " NOT");
 	printf("  %sRAID installation\n",
 		boot.par_1.prompt & FLAG_RAID ? "" : "Non-");
 	if (!boot.par_1.port) printf("  Serial line access is disabled\n");
@@ -679,15 +683,28 @@ static void show_images(char *map_file)
 			}
 		    }
 #endif
+#if 0
 		    if (!descrs.d.descr[image].start_page)
 			printf("    Kernel is loaded \"low\"\n");
 		    else printf("    Kernel is loaded \"high\", at 0x%08lx\n",
 			  (unsigned long) descrs.d.descr[image].start_page*
 			  PAGE_SIZE);
+#else
+		    if (!(descrs.d.descr[image].flags & FLAG_LOADHI))
+			printf("    Kernel is loaded \"low\"\n");
+		    else printf("    Kernel is loaded \"high\"\n");
+#endif
+#if 1
 		    if (!*(unsigned long *) descrs.d.descr[image].rd_size)
 			printf("    No initial RAM disk\n");
 		    else printf("    Initial RAM disk is %ld bytes\n",
 			  *(unsigned long *) descrs.d.descr[image].rd_size);
+#else
+		    if (!descrs.d.descr[image].rd_size)
+			printf("    No initial RAM disk\n");
+		    else printf("    Initial RAM disk is %d sectors\n",
+			  (int)descrs.d.descr[image].rd_size);
+#endif
 		}
 		if (!geo_find(&geo,descrs.d.descr[image].start)) {
 		    printf("    Map sector not found\n");
@@ -705,18 +722,29 @@ static void show_images(char *map_file)
 			printf("    No fallback\n");
 		    else printf("    Fallback: \"%s\"\n",buffer+2);
 		}
+#define OTHER 0
+#if OTHER
 		if (flags & FLAG_KERNEL)
+#endif
 		    if (!geo_find(&geo,addr[1]))
 			printf("    Options sector not found\n");
 		    else {
 			if (read(fd,buffer,SECTOR_SIZE) != SECTOR_SIZE)
 			    die("Read on map file failed (access conflict ?)");
-			printf("    Options: \"%s\"\n",buffer);
+			if (*buffer) printf("    Options: \"%s\"\n",buffer);
+			else printf("    No options\n");
 		    }
-		else if (geo_find(&geo,addr[3])) show_other(fd);
-		   else printf("    Image data not found\n");
+#if OTHER
+		else {
+#else
+		if (!(flags & FLAG_KERNEL)) {
+#endif
+		    if (geo_find(&geo,addr[3])) show_other(fd);
+		    else printf("    Image data not found\n");
+		}
 	    }
 	}
+#undef OTHER
     (void) close(fd);
 #if 0
     checksum = INIT_CKS;
@@ -743,7 +771,7 @@ static void usage(char *name)
       "[ -v N | -v ... ]\n",name);
     fprintf(errstd,"%7s%s [ -C config_file ] [ -b boot_device ] [ -c ] "
       "[ -g | -l | -L ]\n","",name);
-    fprintf(errstd,"%12s[ -i boot_loader ] [ -m map_file ] [ -d delay ]\n","");
+    fprintf(errstd,"%12s[ -F ] [ -i boot_loader ] [ -m map_file ] [ -d delay ]\n","");
     fprintf(errstd,"%12s[ -v N | -v ... ] [ -t ] [ -s save_file | -S save_file ]\n",
       "");
     fprintf(errstd,"%12s[ -p ][ -P fix | -P ignore ] [ -r root_dir ] [ -w ]\n","");
@@ -755,6 +783,7 @@ static void usage(char *name)
     fprintf(errstd,"%7s%s -A /dev/XXX [ N ]\t\tactivate a partition\n","",name);
     fprintf(errstd,"%7s%s -M /dev/XXX [ mbr_file ]\tinstall master boot record\n","",name);
     fprintf(errstd,"%7s%s -T help \t\t\tlist additional options\n", "", name);
+    fprintf(errstd,"%7s%s -X\t\t\t\tinternal compile-time options\n", "", name);
     fprintf(errstd,"%7s%s -V [ -v ]\t\t\tversion information\n\n","",name);
     exit(1);
 }
@@ -764,6 +793,7 @@ int main(int argc,char **argv)
 {
     char *name,*reboot_arg,*identify,*ident_opt,*new_root;
     char *tell_param, *uninst_dev, *param, *act1, *act2, ch;
+static char *bitmap_file;
     int query,more,version,uninstall,validate,activate,instmbr,geom;
     struct stat st;
     int fd;
@@ -783,7 +813,7 @@ int main(int argc,char **argv)
 	argc--;
       /* first those options with a mandatory parameter */
       /* Notably absent are "RuUv" */
-	if (strchr("AbCdDfiImMPrsSTx", ch=(*argv)[1])) {
+	if (strchr("AbBCdDEfiImMPrsSTx", ch=(*argv)[1])) {
 	    if ((*argv)[2]) param = (*argv)+2;
 	    else {
 		param = *++argv;
@@ -807,6 +837,9 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 	    case 'b':
 		cfg_set(cf_options,"boot",param,NULL);
 		break;
+	    case 'B':
+		cfg_set(cf_options,"bitmap",param,NULL);
+		break;
 	    case 'c':
 		cfg_set(cf_options,"compact",NULL,NULL);
 		compact = 1;
@@ -820,9 +853,16 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 	    case 'D':
 		cfg_set(cf_options,"default",param,NULL);
 		break;
+	    case 'E':
+	        eflag=1;
+	        bitmap_file = param;
+	        break;
 	    case 'f':
 		cfg_set(cf_options,"disktab",param,NULL);
 		break;
+	    case 'F':
+		force_fs=1;
+		break;
 	    case 'g':
 		geometric |= 1;
 		break;
@@ -850,10 +890,12 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 	    case 'M':
 		instmbr = 1;
 		act1 = param;
+#ifndef LCF_BUILTIN
 		if (argc && argv[1][0] != '-') {
 		    act2 = *++argv;
 		    argc--;
 		}
+#endif
 		break;
 	    case 'p':
 		passw = 1;	/* force re-gen of password file */
@@ -954,26 +996,30 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 		  sizeof(BOOT_PARAMS_1),
 		  sizeof(BOOT_PARAMS_2)
 		     );
-#else
-		printf(
-		"-DIMAGES=%d "
-		"-DCODE_START_1=%d -DCODE_START_2=%d "
-		  "-DDESCR_SIZE=%d "
-		  "-DDSC_OFF=%d -DDSC_OFF2=%d -DDFCMD_OFF=%d -DMSG_OFF=%d "
-		  "-DFLAGS_OFF=%d"
-		  "\n"
-		  ,
-		  MAX_IMAGES,
-		  sizeof(BOOT_PARAMS_1),
-		  sizeof(BOOT_PARAMS_2),
-		  sizeof(IMAGE_DESCR),
-		  (void *) &dummy.par_1.descr[0]-(void *) &dummy,
-		  (void *) &dummy.par_1.descr[1]-(void *) &dummy,
-		  (void *) &dummy.par_1.descr[2]-(void *) &dummy,
-		  (void *) &dummy.par_1.msg_len-(void *) &dummy,
-		  (void *) &dummy2.flags-(void *) &dummy2
-		     );
+#if 1
+{
+#include "bitmap.h"
+	RGB pal[16];
+	RGB2 pal2[16];
+	
+	printf("\nBMFH=%d BMH=%d BMH2=%d RGB=%d(%d) RGB2=%d(%d) LH=%d\n",
+	sizeof(BITMAPFILEHEADER),
+	sizeof(BITMAPHEADER),
+	sizeof(BITMAPHEADER2),
+	sizeof(RGB),
+	sizeof(pal),
+	sizeof(RGB2),
+	sizeof(pal2),
+	sizeof(BITMAPLILOHEADER) );
+	printf("\nBPB=%d BP_DOS=%d\n",
+		sizeof(BIOS_PARAMETER_BLOCK),
+		sizeof(BOOT_PARAMS_DOS)
+		  );
+}
 #endif
+#endif
+		printf("\nCFLAGS = " CFLAGS "\n");
+		printf("\nMAX_IMAGES = %d\n", MAX_IMAGES);
 		exit(0);
 	    case 'z':
 		zflag++;	/* force zero of MBR 8-byte area */
@@ -1001,11 +1047,11 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    return 0;
 	}
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
-	       "Development beyond version 21 Copyright (C) 1999-2001 John Coffman\n"
+	       "Development beyond version 21 Copyright (C) 1999-2002 John Coffman\n"
 	       );
         if (verbose>0) printf("Released %s and compiled at %s on %s.\n",
 	        VERSION_DATE, __TIME__, __DATE__);
-#ifdef LCF_DSECS
+#if 0 && defined(LCF_DSECS)
 	printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 #endif
         printf("\n");
@@ -1014,9 +1060,13 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
     preload_types();
     if (geometric & (geometric-1))
 	die ("Only one of '-g', '-l', or '-L' may be specified");
+
+    if (verbose > 0) errstd = stdout;
+    if (eflag) do_bitmap_edit(bitmap_file);
+
     fd = cfg_open(config_file);
     more = cfg_parse(cf_options);
-    if (verbose > 0) errstd = stdout;
+
     if (verbose>=6) printf("main: cfg_parse returns %d\n", more);
     if (tell_param) probe_tell(tell_param);
     if (instmbr) do_install_mbr(act1, act2);
@@ -1072,6 +1122,10 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	lba32 = 1;
     }
 #endif
+    if (linear && !nowarn) fprintf(errstd,
+    	"Warning: LINEAR is deprecated in favor of LBA32:  LINEAR specifies a 24-bit\n"
+	"  address; whereas LBA32 specifies a 32-bit address; use LINEAR only if you\n"
+	"  REALLY know what you are doing.\n");
     if (verbose<0 && cfg_get_strg(cf_options,"verbose"))
 	verbose = to_number(cfg_get_strg(cf_options,"verbose"));
     if (verbose<0) verbose = 0;
@@ -1081,7 +1135,7 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 /* test for a RAID installation */
     raid_offset = raid_setup();
     if (verbose >= 2)
-        printf("raid_setup returns offset = %08lX\n", raid_offset);
+        printf("raid_setup returns offset = %08lX  ndisk = %d\n", raid_offset, ndisk);
 	    
 	if (uninstall)
 	    bsect_uninstall(uninst_dev ? uninst_dev : cfg_get_strg(cf_options,
@@ -1104,11 +1158,16 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	if (verbose >=2 && do_md_install)
 	    printf("raid flags: at bsect_open  0x%02X\n", raid_flags);
 
-	bsect_open(cfg_get_strg(cf_options,"boot"),cfg_get_strg(cf_options,"map") ?
-	  cfg_get_strg(cf_options,"map") : MAP_FILE,cfg_get_strg(cf_options,
-	  "install"),cfg_get_strg(cf_options,"delay") ? to_number(cfg_get_strg(
-	  cf_options,"delay")) : 0,cfg_get_strg(cf_options,"timeout") ?
-	  to_number(cfg_get_strg(cf_options,"timeout")) : -1, raid_offset);
+	bsect_open(
+		cfg_get_strg(cf_options,"boot"),
+		cfg_get_strg(cf_options,"map") ?
+			cfg_get_strg(cf_options,"map") : MAP_FILE,
+		cfg_get_strg(cf_options,"install"),
+		cfg_get_strg(cf_options,"delay") ?
+			to_number(cfg_get_strg(cf_options,"delay")) : 0,
+		cfg_get_strg(cf_options,"timeout") ?
+			to_number(cfg_get_strg(cf_options,"timeout")) : -1,
+		raid_offset );
 	if (more) {
 	    cfg_init(cf_top);
 	    if (cfg_parse(cf_top)) cfg_error("Syntax error");
