@@ -1,7 +1,7 @@
 /* lilo.h  -  LILO constants */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -78,6 +78,7 @@ source directory.
 #define MAX_RAID	30	/* max number of RAID disks in a set */
 #define MAX_TOKEN	255 /* max device Token length */
 #define MAX_IMAGE_NAME	15 /* maximum name length (w/o terminating NUL) */
+#define MAX_DESCR_SECTORS  12  /* upper limit on MAX_DESCR_SECS */
 
 #ifdef LCF_PASS160
 #define SHS_PASSWORDS		/* use this one if SHS passwords are in use */
@@ -100,6 +101,9 @@ source directory.
 #else
 #define MAX_DESCR_SECS 2	/* maximum # of descriptor sectors (8 max) */
 #endif
+#if MAX_DESCR_SECS > MAX_DESCR_SECTORS
+# error "Maximum DSECS=x exceeded."
+#endif
 #define MAX_IMAGES      ((SECTOR_SIZE*MAX_DESCR_SECS-sizeof(long)-1)/sizeof(IMAGE_DESCR))
 			  /* maximum number of images */
 
@@ -204,6 +208,7 @@ source directory.
 #define FLAG_RAID_NOWRITE 8	/* defeat RAID writeback of command line */
 #define FLAG_NOBD	16	/* defeat BIOS data collection at boot time */
 #define FLAG_SAVE (FLAG_PROMPT|FLAG_NOBD)  /* All but raid flags */
+#define FLAG_LARGEMEM	32	/* BIOS has MoveExtMemBlk support for 386 */
 
 /* these are the descriptor flags */
 #define FLAG_VGA	1	/* override VGA mode */
@@ -214,6 +219,7 @@ source directory.
 #define FLAG_SINGLE	32	/* single key activation */
 #define FLAG_FALLBACK	64	/* fallback command line exists */
 #define FLAG_PASSWORD	128	/* this image requires a password */
+#define FLAG_LOADHI	256	/* this kernel loads high (>=1Mb) */
 
 #define VGA_NOCOVR	0x8000	/* VGA setting not overridden on command line */
 
@@ -228,7 +234,8 @@ source directory.
 #define NEW_HDR_SIG	"HdrS"	/* setup header signature */
 #define NEW_HDR_VERSION	0x200	/* header version number */
 #define NEW2_HDR_VERSION 0x202	/* new cmdline protocol */
-#define LOADER_VERSION	1	/* loader version, for SETUP_HDR.loader */
+#define NEW3_HDR_VERSION 0X203	/* defines CL_RAMDISK_MAX */
+#define LOADER_VERSION	0x02	/* loader version, for SETUP_HDR.loader */
 #define LFLAG_HIGH	1	/* SETUP_HDR.flags */
 #define LFLAG_USE_HEAP	0x80
 
@@ -263,6 +270,7 @@ SYSSEG    = DEF_SYSSEG			! system loaded at 0x10000 (65536).
 MAX_SECONDARY_asm1 = MAX_SECONDARY_FILESIZE + SECTOR_SIZE - 1
 MAX_SECONDARY_asm = MAX_SECONDARY_asm1 / SECTOR_SIZE	! define for the assembler
 MAX_DESCR_SECS_asm = MAX_DESCR_SECS	! **
+MAX_DESCR_SECTORS_asm = MAX_DESCR_SECTORS ! **
 MAX_IMAGE_NAME_asm = MAX_IMAGE_NAME	! **
 MAX_PW_CRC_asm	= MAX_PW_CRC		! **
 SECTOR_SIZE_asm = SECTOR_SIZE		! **
@@ -270,9 +278,9 @@ MAX_MENU_TITLE_asm = MAX_MENU_TITLE	! **
 
 MAPcode   = MAX_SECONDARY_asm*0x200	! size of secondary code
 MAP	  = MAPcode+0x200		! plus space for cmdline input
-DESCR	  = MAP+0x200	     ! 0x2200	! descriptor load area offset
-DFLCMD	  = DESCR+0x200*MAX_DESCR_SECS_asm	     ! 0x2600	! default command line
-KEYTABLE  = DFLCMD+0x200     ! 0x2800	! keyboard translation table offset
+DFLCMD	  = MAP+0x200			! default command line
+DESCR	  = DFLCMD+0x200			! descriptor load area offset
+KEYTABLE  = DESCR+0x200*MAX_DESCR_SECS_asm	     ! 0x2600	! default command line
 DATAEND	  = KEYTABLE+0x200   ! 0x2A00	! PARAMLINE or Load-Low area
 
 STACKSEG  = 0x9000	     ! MUST == INITSEG for kernel 2.0.36 (and others?)
@@ -351,6 +359,7 @@ CL_HEADER_ID	= 0x202			! "HdrS"
 CL_HDRS_VERSION	= 0x206			! 0x0201=old;  0x0202=new
 NEW_VERSION	= NEW2_HDR_VERSION	! 0x0202 for new cmdline protocol
 CL_POINTER	= 0x228			! new pointer is dword address
+CL_RAMDISK_MAX	= CL_POINTER+4		! ramdisk_max; header version 0x0203
 
 
 #endif
