@@ -25,6 +25,7 @@ source directory.
 #include "device.h"
 #include "geometry.h"
 #include "partition.h"
+#include "bsect.h"
 #include "bdata.h"
 #include "probe.h"
 /*#include "lrmi.h"*/
@@ -72,6 +73,7 @@ static void do_ebda(void);
 static void do_cr_pr(void);
 static void do_help(void);
 static void do_geom(char *bios);
+static void do_bitmap(char *file);
 static void do_geom_all(void);
 static void do_table(char *part);
 #if VIDEO
@@ -85,7 +87,7 @@ extern CHANGE_RULE *change_rules;	/* defined in partition.c */
 
 #if 0   /* def LCF_ALL_PARTITIONS */
 
-#define SECTORSIZE ((long long)SECTOR_SIZE)
+#define LLSECTORSIZE ((long long)SECTOR_SIZE)
 
        _syscall5(int,  _llseek,  uint,  fd, ulong, hi, ulong, lo,
        loff_t *, res, uint, wh);
@@ -111,11 +113,12 @@ struct Probes {
 	}
 	list[] = {
 { "help",  do_help,  NULL,  "Print list of -T(ell) options"	},
+{ "bitmap=",do_bitmap,"<file>", "Display .bmp file X,Y/color/timer information"},
 { "ChRul", do_cr_pr, NULL,  "List partition change-rules"  },
 { "EBDA",  do_ebda,  NULL,  "Extended BIOS Data Area information" },
 { "geom=", do_geom,  "<bios>", "Geometry CHS data for BIOS code 0x80, etc." },
 { "geom" , do_geom_all, NULL, "Geometry for all BIOS drives" },
-{ "table=", do_table, dev,   "Partition table information for /dev/hda, etc." },
+{ "table=", do_table, dev, "Partition table information for /dev/hda, etc."},
 #if VIDEO
 { "video", do_video, NULL,  "Graphic mode information" },
 #endif
@@ -349,9 +352,11 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
       
       if ((dp->info) & EDD_PARAM_GEOM_VALID) {
          if ((geom->n_sect != dp->sectors ||
-             geom->n_head != dp->heads) && !nowarn && !(warned[drive-0x80]++))
+             geom->n_head != dp->heads) && !nowarn && !(warned[drive-0x80]&1) ) {
                 fprintf(errstd,"Warning: Int 0x13 function 8 and function 0x48 return different\n"
                                "head/sector geometries for BIOS drive 0x%02X\n", drive);
+		warned[drive-0x80] |= 1;
+	    }
          geom->n_cyl  = dp->cylinders;
          geom->n_head = dp->heads;
          geom->n_sect = dp->sectors;
@@ -387,14 +392,19 @@ static void do_ebda(void)
     printf("    Conventional Memory = %dK    0x%06X\n", m, m<<10);
     m <<= 10;
     m -= 0x200;
+#ifndef LCF_BUILTIN
     n = m - (MAX_SECONDARY+4+MAX_DESCR_SECS)*SECTOR_SIZE;
+#else
+    n = (select_loader()->size + SECTOR_SIZE - 1) / SECTOR_SIZE;
+    n = m - (n+4+MAX_DESCR_SECS)*SECTOR_SIZE;
+#endif
 #if 0
     if (!(verbose>0)) return;
 #else
     printf("\n");
 #endif
-    printf("    The First stage loader boots at:  0x%08X  (%04X:0000)\n",
-    			FIRSTSEG<<4, FIRSTSEG);
+    printf("    The First stage loader boots at:  0x%08X  (0000:%04X)\n",
+    			FIRSTSEG<<4, FIRSTSEG<<4);
     printf("    The Second stage loader runs at:  0x%08X  (%04X:%04X)\n",
     			n, n>>4, n&15);
     printf("    The kernel cmdline is passed at:  0x%08X  (%1X000:%04X)\n",
@@ -526,7 +536,7 @@ static void do_table(char *part)
     }
     i=5;
     while (verbose>0 && base) {
-        if (llseek(fd, SECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
+        if (llseek(fd, LLSECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
             die("secondary llseek failed");
 	if (read(fd, pt, sizeof(pt)) != sizeof(pt)) die("secondary read pt failed");
 	if ( read(fd, &boot_sig, sizeof(boot_sig)) != sizeof(boot_sig)  ||
@@ -717,7 +727,7 @@ void probe_tell (char *cmd)
     }
     printf("Unrecognized option to '-T' flag\n");
     do_help();
-    
+    printf("\n");
     exit(1);
 }
 
@@ -832,8 +842,24 @@ int bios_device(GEOMETRY *geo, int device)
     		match, match==1 ? "" : "es", bios1);
 
     if (match != 1) bios1 = -1;
+    else {	/* match == 1 */
+	get_geom(bios1, &bdata);
+	if (!nowarn &&
+	    (  (geo->sectors && geo->sectors!=bdata.n_sect)  ||
+	       (geo->heads && geo->heads!=bdata.n_head)  ) &&
+	    !(warned[bios1-0x80]&2) ) {
+	    fprintf(errstd,"Warning: Kernel & BIOS return differing CHS geometry for device 0x%02X\n", bios1);
+	    warned[bios1-0x80] |= 2;
+	}
+    }
 
     return (geo->device = bios1);
 }
 
 
+static void do_bitmap(char *file)
+{
+    printf("Color, Positioning, and Timer information for file:  %s\n", file);
+    printf("...<unimplemented>...\n");
+}
+
